@@ -6,19 +6,17 @@ Apache2::Controller - framework for Apache2 handler apps
 
 =head1 VERSION
 
-Version 0.0.4 - BETA TESTING
+Version 1.000.000 - FIRST RELEASE
 
 =cut
 
-use strict;
-use warnings FATAL => 'all', NONFATAL => 'redefine';
-
-our $VERSION = '0.0004';
+use version;
+our $VERSION = version->new('1.000.000');
 
 =head1 SYNOPSIS
 
 For Apache2 config file setup see L<Apache2::Controller::Dispatch>,
-which pushes a PerlResponseHandler of Apache::Controller, which
+which sets a PerlResponseHandler of Apache::Controller, which
 then instantiates your controller object and calls the chosen
 method for the uri.
  
@@ -29,8 +27,7 @@ method for the uri.
  use base qw( Apache2::Controller );
 
  use Apache2::Const -compile => qw( :http );
- use Readonly;
- Readonly our @ALLOWED_METHODS => qw( default bar baz );
+ sub allowed_methods {qw( default bar baz )}
 
  # suppose '/foo' is the uri path dispatched to this controller
  # and your dispatch uses L<Apache2::Controller::Dispatch::Simple>
@@ -45,13 +42,14 @@ method for the uri.
 
  # http://myapp.xyz/foo/bar/biz/schnozz
  sub bar {
-     my ($self) = @_;
-     my @path_args = @{ $self->{path_args} }; # qw( biz schnozz )
+     my ($self, @path_args) = @_;             
+     # @path_args is:
+     #      qw( biz schnozz )
+     #      @{ $self->{path_args} }
+     #      @{ $self->pnotes->{path_args} }
+
      $self->content_type('text/html');
-     $self->print(q{
-        <p>Sometimes you eat the bear, 
-        and sometimes the bear eats you.</p>
-     });
+     $self->print(q{ <p>"WE ARE ALL KOSH"</p> });
      return Apache2::Const::HTTP_OK;
  }
 
@@ -71,7 +69,7 @@ method for the uri.
      return Apache2::Const::HTTP_OK;
  }
 
-1;
+ 1;
 
 You could implement a pretty nice REST interface, or any other kind
 of HTTP-based API, by returning the appropriate HTTP status codes.
@@ -85,12 +83,13 @@ for your controller class to render HTML with L<Template> Toolkit.
 Apache2::Controller is a lightweight controller framework for 
 object-oriented applications designed to run only under mod_perl 
 children in high-performance Apache2 handler modules.  It features URL 
-dispatch with flexible configuration, auth plugins for OpenID, a cookie 
-session and MySQL store, a pluggable infrastructure for your own models 
-and views, and base inheritance configuration allowing you to 
+dispatch with flexible configuration, auth plugins, a cookie tracker
+for Apache::Session, liberty for any storage models that work under mod_perl,
+rendering using Template Toolkit or direct printing with Apache,
+and base inheritance configuration allowing you to 
 construct your applications as you need, without trying to be all things 
-to all people or assimilate the world, and without having to load all 
-site modules for every page handler. It is intended as a framework for 
+to all people or assimilate the world.  
+It is intended as a framework for 
 new applications specialized as Apache2 handlers, not as a means to 
 absorb existing applications or to create portable code.  
 
@@ -163,12 +162,15 @@ L<Apache2::RequestRec> object C<<$r>>,
 it is in C<<$self->{r}>>.
 
 Some other request phase handlers register later-stage handlers,
-for example to save the session with a C<PerlCleanupHandler>
-after the controller successfully completes the C<Response> phase.
+for example to save the session or rollback the database 
+transaction with C<PerlLogHandler>'s
+after the connection output is complete.
 
-These handlers for other stages will return DECLINED or DONE if 
-necessary to prevent running your Apache2::Controller in the
-case of an error.
+The controller handler always returns your set HTTP status code
+or OK (0) to Apache.  In general you return the status code
+that you want to set, or return OK.  I don't know what happens
+if you return DONE or DECLINED.  See L<Apache2::Const>
+and L<Apache2::Controller::Refcard>.
 
 Add handlers in your config file with your own modules which 
 C<use base> to inherit from these classes as you need them:
@@ -176,7 +178,7 @@ C<use base> to inherit from these classes as you need them:
 =head2 PerlHeaderParserHandler Apache2::Controller::Session
 
 C<<$r->pnotes->{session}>> fed from and stored to an 
-L<Apache::Session> tied hash.  Pushes a PerlCleanupHandler
+L<Apache::Session> tied hash.  Pushes a PerlLogHandler
 to save the session after the main controller returns OK.
 
 See L<Apache2::Controller::Session>
@@ -206,7 +208,7 @@ See L<Apache2::Controller::Authz::Groups>.
 
 =head1 Apache2::Controller RESPONSE PHASE HANDLER
 
-Apache2::Controller is pushed as the PerlResponseHandler if 
+Apache2::Controller is set as the PerlResponseHandler if 
 the dispatch class finds a valid module and method for the request.
 Your controller uses
 
@@ -273,24 +275,62 @@ and then just return, Apache2::Controller will not override the set status.
 See L<Apache2::Controller::Refcard> for a list of HTTP return constants
 and corresponding numbers and messages.
 
+=head2 REDIRECTS, ETC.
+
+For external redirects,
+set C<< $self->err_headers_out->add(Location => 'http://foo.bar') >>
+and return C<< $self->status(Apache2::Const::REDIRECT) >>.  C<< $self >>
+is (delegates to) the L<Apache2::Request> object which inherits from 
+L<Apache2::RequestRec> and friends.  You have access to everything
+without dereferencing pointers, for instance, internal redirects
+that process a new request within a subprocess of the current one.
+(This dereferences C<< $self->{r} >> internally but it makes your
+controller logic more simple and direct.)
+
+Keep in mind that if you use other request phase processors that
+push a C<PerlLogHandler> like 
+L<Apache2::Controller::DBI::Connector> or
+L<Apache2::Controller::Session>, those will still run.
+So in the case of a redirect while you have an open transaction,
+or where you do not want to save tied session changes when redirecting,
+you should trap errors with C<< eval >> and deal with it accordingly.
+
+You should also not fiddle with the connection by causing
+Apache2 to close it prematurely, else these handlers may not
+run synchronously before another request is received that
+may have depended on their behavior.
+
 =head1 ERRORS
 
-L<Apache2::Controller::X> is the L<Exception::Class> hierarchy used
-to kick up errors and redirects.  You can use or subclass this,
-or you can throw your own exceptions with L<Exception::Class> 
-or another exception object, or just C<die()>, but you either 
-die with an Apache2::Controller::X subclass with an status 
-field or should not die and return an HTTP status code instead.
+If you decide to set an error status code, you can print your
+own content and return that status code.
+
+If you want to use error templates, 
+barf L<Apache2::Controller::X> objects.  These print a stack trace
+to the error log at the WARN level of L<Log::Log4perl> from
+this module's namespace.  If errors crop up from
+other A2C request phase handlers, try setting
+WARN log level for L<Apache2::Controller::NonResponseBase>
+or L<Apache2::Controller::NonResponseRequest>.
+
+Also see L<Apache2::Controller::Render::Template>.
+
+You can use or subclass L<Apache2::Controller::X>,
+to use C<< a2cx() >>,
+or you can throw your own exception objects,
+or just C<< die() >>, or C<< croak() >>,
+or set C<< $self->status >>, headers etc., possibly printing content, 
+or return the appropriate status from your controller method.
 
 See L<Apache2::Controller::X> for help on throwing exceptions
-and external redirects.
+with HTTP status, data dumps, etc.
 
 If your code does break, die or throw an exception, this is 
 caught by Apache2::Controller.  If your controller module implements
 an C<error()> method, for instance by use of the base
 L<Apache2::Controller::Render::Template> which looks in
 template_dir/errors/ for the appropriately named error template,
-then C<$handler->error()> will be called passing the $EVAL_ERROR
+then C<$handler->error()> will be called passing the C<< $EVAL_ERROR >>
 or exception as the first argument.
 
  package MyApp::C::Foo;
@@ -318,7 +358,7 @@ See L<Apache2::Controller::Session/ERRORS> for how to control
 whether or not a session is saved.  Usually it is automatically
 saved.  If you don't want it saved when you have an error
 in your controller, you'll have to use C<<eval>> and then
-set C<<$self->notes->{connection_closed}>> before you return
+set C<<$self->notes->{a2c_connection_aborted}>> before you return
 the error code.
 
 =head1 CONTROLLER CLOSURES
@@ -336,22 +376,6 @@ to something specific if you need that.  Otherwise it will let
 mod_perl set it to whatever it chooses when you start to print.
 This is usually text/html.
 
-=head1 REDIRECTS
-
-Use or subclass Apache2::Controller::X, and then throw an 
-Apache2::Controller::X::Redirect or your Exception::Class
-object which isa Apache2::Controller::X::Redirect.
-
- Apache2::Controller::X::Redirect->throw("http://perl.apache.org");
-
-Apache2::Controller::handler() will trap the exception, terminate
-the response chain with C<DONE>, and send redirects on their way.  
-See L<Apache2::Controller::X>.
-
-If you don't want to terminate the response chain, maybe you can set
-C<<$self->status(REDIRECT)>> and C<<$self->headers_out->{Location}>>...
-so the session would still be saved, for example.  Experiment.
-
 =head1 LOGGING
 
 Apache2::Controller uses L<Log::Log4perl>.  See that module
@@ -386,24 +410,38 @@ child interpreter.  So,
 you can use package namespace effectively to store data
 that will persist in the mod_perl child across requests.
 
-=head1 WARNINGS AND CAVEATS
+=head1 LOAD BALANCING
 
-=head2 Handle errors appropriately
+A2C does not have to load 
+all site modules for every page handler, which could help with load-balancing
+highly optimized handlers for specific URI's while having a universal
+application installer.  
 
-If you return an error code of 400 HTTP_BAD_REQUEST or higher,
-Apache2::Controller will return DONE instead of OK, so Apache2
-will not process any further handlers in the stack but will
-end the request right here.
+Picture if you will, a programming utopia in which all engineers
+are respected, highly paid and content.  
 
-So, it's up to you in your controllers to use C<eval { }> appropriately
-and deal with errors, for instance, if your database transaction
-goes awry, it's up to you to roll back a transaction in the 
-controller module.  Don't push a PerlCleanupHandler to do that,
-though it's tempting to do so, because if you return an error code
-from the controller method, the PerlCleanupHandler will not run.
+You deploy the same Apache, the
+same CPAN modules and your whole application package to every server,
+and attach a url-generating subroutine to the L<Template|Template> stash
+that puts in a different hostname when the URI is one of your
+load-intensive functions.  
+
+ <a href="[% myurl('/easy') %]">easy</a> "http://pony.x.y/easy"
+
+ <a href="[% myurl('/hard') %]">hard</a> "http://packhorse.x.y/hard"
+
+Web designers can be taught to use this function C<< myurl() >>, 
+but system admins
+maintain the map that it loads to figure out what servers to use.
+
+Then the Apache2 config files on those
+packhorse servers would pre-load only the subclassed controllers
+that you needed, and redirect all other uri requests to the pony servers.
 
 =cut
 
+use strict;
+use warnings FATAL => 'all';
 use English '-no_match_vars';
 
 use base qw( Apache2::Request Apache2::Controller::Methods );
@@ -419,6 +457,7 @@ use URI;
 use HTML::Entities;
 use URI::Escape;
 use HTTP::Status qw( status_message );
+use Scalar::Util qw( looks_like_number );
 
 use Apache2::Controller::X;
 use Apache2::Controller::Funk qw( check_allowed_method log_bad_request_reason );
@@ -433,166 +472,6 @@ use Apache2::Const -compile => qw( :common :http );
 
 =head1 FUNCTIONS
 
-=head2 handler
-
- Apache2::Controller::handler( Apache2::RequestRec object )
-
-The handler is pushed from an Apache2::Controller::Dispatch
-subclass and should not be set in the config file.  It looks
-for the controller module name in C<< $r->notes->{controller} >>
-and for the method name in C<< $r->notes->{method} >>.
-
-It runs start_request() if implemented, then runs the
-selected method, then runs end_request() if implemented,
-setting the status to any positive return value
-returned by any of those methods.  (The last positive
-value returned will be the final status.)
-
-Errors are intercepted and if the handler object was created
-and implements an C<< $handler->error($exception) >> method 
-then the exception will be passed as the argument.
-
-An HTTP status code of HTTP_BAD_REQUEST or greater will 
-cause log_reason to be called with a truncated error string
-and the uri for recording in the access log.
-
-=head3 RUN_ALL mode
-
-In the odd chance you plan to push further PerlResponseHandlers
-from your controller or server config, maybe to print
-debugging data at the end in HTML comments for instance, 
-then you need to set a boolean directive (temporarily a C<PerlSetVar>
-variable)
-called C<A2CRunAllResponseHandlers>. Then C<Apache2::Controller>
-will check Apache's stack of PerlResponseHandlers for the Response
-phase of the request lifecycle, to see whether you have pushed
-more response handlers to the stack, and if so will run them.
-Otherwise, the Response phase will stay in default RUN_FIRST 
-mode and no pushed PerlResponseHandlers will be executed.
-
-=cut
-
-my %supports_error_method = ( );
-
-sub handler : method {
-    my ($class, $r) = @_;
-    return $class if !defined $r;
-
-    my $method = $r->notes->{method};
-
-    DEBUG("$class -> $method");
-
-    my ($handler, $run_all, $status, $X) = ( );
-    eval {
-
-        $handler = $class->new($r);
-        $run_all = $handler->get_directive('A2CRunAllResponseHandlers');
-        $method  = $handler->{method};
-
-        DEBUG("executing $class -> $method()");
-        $status = $handler->$method();
-        $status = $handler->status() if !defined $status;
-    };
-    if ($X = $EVAL_ERROR) {
-        my $ref = ref($X);
-
-        if (blessed($X) && $X->can('status')) {
-            $status = $X->status;
-        }
-        $status = Apache2::Const::SERVER_ERROR if !defined $status;
-
-        $status = Apache2::Const::HTTP_OK 
-            if $status == Apache2::Const::OK;
-
-        # if a redirect, just set the location and status
-        if ($ref && $X->isa('Apache2::Controller::X::Redirect')) {
-            $status = Apache2::Const::REDIRECT;
-            $r->err_headers_out->add(Location => "$X");
-        }
-
-        # else process the error
-        else {
-
-            # if appropriate and able to call self->error(), do that now
-            if ($handler && !$r->notes->{use_standard_errors}) {
-
-                eval {
-                    if (exists $supports_error_method{$class}) {
-                        $handler->error($X);
-                    } 
-                    elsif ($class->can('error')) {
-                        $handler->error($X); 
-                    }
-                };
-                if (my $tempX = Exception::Class->caught('Apache2::Controller::X')) {
-                    $X = $tempX;
-                }
-                elsif ($EVAL_ERROR) {
-                    $X = "$EVAL_ERROR";
-                }
-            }
-
-            # now decide how to output debugging log:
-            if (ref($X) && $X->isa('Apache2::Controller::X')) {
-                DEBUG(sub {
-                    ref($X).": $X\n".($X->dump ? Dump($X->dump) : '').$X->trace()
-                });
-            }
-            else {
-                DEBUG("Caught an unknown error: $X");
-            }
-            INFO("Error caught in Apache2::Controller::handler: $X\n");
-        }
-    }
-
-    DEBUG("done with handler processing...");
-    DEBUG(sub {
-        my $ctype = $r->content_type();
-        "content type is ".($ctype || '[undef]');
-    });
-    
-    $status ||= Apache2::Const::HTTP_OK;
-    $r->status($status);
-
-    DEBUG(sub { 
-        "status: $status ".status_message($status)
-        .", status_line for ".$r->status." is '".$r->status_line()."'"
-    });
-
-    # if status is an erorr, file error (possibly truncated) as a 
-    # log_reason in the access log for why this request was denied.
-    if ($status >= Apache2::Const::HTTP_BAD_REQUEST) {
-        log_bad_request_reason($r, $X);
-        $r->status_line("$status ".status_message($status)) 
-            if !$r->status_line;
-        return Apache2::Const::OK;
-    }
-    elsif ($run_all) {
-        # return OK or DECLINED depending on whether the controller module
-        # or apache config pushed any further handlers.  essentially switch
-        # further PerlResponseHandlers from RUN_FIRST to RUN_ALL mode.
-        my @resp_handlers = map { eval { $_->() } } 
-            @{ $r->get_handlers('PerlResponseHandler') || [ ] };
-
-        my $last_resp_h = $resp_handlers[-1];
-
-        return defined $last_resp_h && $last_resp_h eq $class
-            ? Apache2::Const::OK
-            : Apache2::Const::DECLINED;
-        
-        return Apache2::Const::OK;
-    }
-    else {
-        return Apache2::Const::OK;
-    }
-
-    # supposedly you can return the http status, but it doesn't work right
-    # if you return HTTP_OK.  shouldn't it?
-    # i prefer to directly control what is happening
-    # by setting the status and then telling Apache whether
-    # to continue or to stop processing the request if not.
-}
-
 =head2 new
 
  $handler = MyApp::C::ControllerSubclass->new( Apache2::RequestRec object )
@@ -617,12 +496,28 @@ starts, you can override the constructor.
          my ($r) = @_;
          my $dbh = $r->pnotes->{dbh};
          $dbh->rollback() unless $r->notes->{commit_success};
-     });
+     });    # implements some convention that you set 'commit_success', say
      return $self;
  }
 
- package MyApp::C::Foo;
- use base qw( Apache2::Controller MyApp::InitController );
+Similarly, to do something always at the end of every 
+request, from within the dispatched PerlResponseHandler:
+
+ package MyApp::DestroyController;
+ use Devel::Size;
+ use Log::Log4perl qw(:easy);
+ my $MAX = 40 * 1024 * 1024;
+ sub DESTROY {
+     my ($self) = @_;
+     my $size = total_size($self);  # whoo baby!
+     INFO("size of $self->{class} is bigger than $MAX!") if $size > $MAX;
+     return; # self is destroyed
+ }
+
+And your subclass is:
+
+ package MyApp::Controller::Foo;
+ use base qw( Apache2::Controller MyApp::InitController MyApp::DestroyController );
 
  # ...
 
@@ -677,7 +572,8 @@ sub new {
     }
 
     DEBUG(sub { Dump({
-        map {($_ => $self->{$_} ? "$self->{$_}" : '[undef]')} keys %$self 
+        # for simple debugging, stringify objects, otherwise this can get huge
+        map {($_ => defined $self->{$_} ? "$self->{$_}" : undef)} keys %$self 
     }) });
 
     return $self;
@@ -685,7 +581,142 @@ sub new {
 
 =head1 METHODS
 
-Handler's methods are extended by L<Apache2::Controller::Methods>.
+Methods are also extended by 
+L<Apache2::Controller::Methods|Apache2::Controller::Methods>.
+
+=head2 handler
+
+ # called from Apache, your subclass pushed to PerlResponseHandler
+ # by your A2C dispatch handler:
+ MyApp::Controller::Foo->handler( Apache2::RequestRec object )
+
+The handler is pushed from an Apache2::Controller::Dispatch
+subclass and via your dispatched subclass of Apache2::Controller.
+It should not be set in the config file.  It looks
+for the controller module name in C<< $r->notes->{controller} >>
+and for the method name in C<< $r->notes->{method} >>.
+
+Errors are intercepted and if the handler object was created
+and implements an C<< $handler->error($exception) >> method 
+then the exception will be passed as the argument.
+
+An HTTP status code of HTTP_BAD_REQUEST or greater will 
+cause log_reason to be called with a truncated error string
+and the uri for recording in the access log.
+
+=cut
+
+my %supports_error_method = ( );
+
+sub handler : method {
+    my ($class, $r) = @_;
+    return $class if !defined $r;
+
+    my $method = $r->notes->{method};
+
+    DEBUG("$class -> $method");
+
+    my ($handler, $status, $X) = ( );
+    eval {
+
+        $handler = $class->new($r);
+        $method  = $handler->{method};
+
+        DEBUG("executing $class -> $method()");
+        my $args = $r->pnotes->{path_args} || [];
+        $status = $handler->$method(@{$args});
+        $status = $handler->status() if !defined $status;
+
+        if (defined $status) {
+            if (ref $status || !looks_like_number($status)) {
+                a2cx message => "Controller returned or set non-numeric status",
+                    status  => Apache2::Const::SERVER_ERROR,
+                    dump    => { controller_set_status => $status };
+            }
+            elsif ($status < 0) {
+                a2cx "controller must set http status >= 0";
+            }
+        }
+
+    };
+    if ($X = $EVAL_ERROR) {
+        my $ref = ref($X);
+
+        $status 
+          = $ref && blessed($X) && $X->can('status') 
+                             ? ($X->status || Apache2::Const::SERVER_ERROR)
+          : !defined $status                ? Apache2::Const::SERVER_ERROR
+          : !looks_like_number($status)     ? Apache2::Const::SERVER_ERROR
+          : $status == Apache2::Const::OK   ? Apache2::Const::HTTP_OK
+          : Apache2::Const::SERVER_ERROR;
+
+        # if appropriate and able to call self->error(), do that now
+        if ($handler && !$r->notes->{use_standard_errors}) {
+
+            eval {
+                if (exists $supports_error_method{$class}) {
+                    $handler->error($X);
+                } 
+                elsif ($class->can('error')) {
+                    $handler->error($X); 
+                }
+            };
+            $X = Exception::Class->caught('Apache2::Controller::X')
+                || $EVAL_ERROR 
+                || $X;
+        }
+
+        # now decide how to output debugging log:
+        if (ref($X) && $X->isa('Apache2::Controller::X')) {
+            WARN(sub {
+                ref($X).": $X\n".($X->dump ? Dump($X->dump) : '').$X->trace()
+            });
+        }
+        else {
+            WARN("Caught an unknown error: $X");
+        }
+    }
+
+    DEBUG("done with handler processing...");
+    DEBUG(sub {
+        my $ctype = $r->content_type();
+        "content type is ".($ctype || '[undef]');
+    });
+    
+    $r->status($status) if $status;
+
+    DEBUG(sub { 
+        my $stat  = defined $status ? $status : '';
+        my $sline = $r->status_line() || '[none]';
+        my $smsg  
+            = defined $status 
+            ? (status_message($status) || '[no msg]') 
+            : 'N/A';
+        my $debugstatus = defined $status ? $status : '[status not defined]';
+        "status: $debugstatus ($smsg); status_line='$sline'";
+    });
+
+    if (!defined $status) {
+        return Apache2::Const::OK;
+    }
+    elsif ($status >= Apache2::Const::HTTP_BAD_REQUEST) {
+        # if status is an error, file error (possibly truncated) as a 
+        # log_reason in the access log for why this request was denied.
+        # (is this desirable?)
+        log_bad_request_reason($r, $X);
+        $r->status_line("$status ".status_message($status)) 
+            if !$r->status_line;
+        return $status;
+    }
+    else {
+        return $status == Apache2::Const::HTTP_OK
+            ? Apache2::Const::OK
+            : $status;
+    }
+
+    # supposedly you can return the http status, but it doesn't work right
+    # if you return HTTP_OK.  shouldn't it?
+}
 
 =head1 USING INHERITANCE
 
@@ -708,7 +739,7 @@ they should add a line to call a shared cleanup method.
 
  package MyApp::C::Foo;
  use base qw( Apache2::Controller MyApp::Cleanup );
- my @ALLOWED_METHODS = qw( foo bar );
+ sub allowed_methods {qw( foo bar )}
 
  sub foo {
      # ...
@@ -745,6 +776,11 @@ every time even if they don't do anything, or whatever.  Bleah.
 Implement what you want to implement from the controller methods.
 The framework won't provide you with any more structure.
 
+=head1 EXAMPLES
+
+Browse the source package from CPAN
+and check out t/lib/* and t/conf/extra.conf.last.in.
+
 =head1 SEE ALSO
 
 L<Apache2::Controller::Dispatch>
@@ -753,6 +789,8 @@ L<Apache2::Controller::Uploads>
 
 L<Apache2::Controller::Session>
 
+L<Apache2::Controller::DBI::Connector>
+
 L<Apache2::Controller::Auth::OpenID>
 
 L<Apache2::Controller::X>
