@@ -4,9 +4,18 @@ package Apache2::Controller::Dispatch::Simple;
 
 Apache2::Controller::Dispatch::Simple - simple dispatch mechanism for A2C
 
+=head1 VERSION
+
+Version 1.000.000 - FIRST RELEASE
+
+=cut
+
+use version;
+our $VERSION = version->new('1.000.000');
+
 =head1 SYNOPSIS
 
- <Location /subdir>
+ <Location "/subdir">
      SetHandler modperl
      PerlInitHandler MyApp::Dispatch
  </Location>
@@ -15,22 +24,20 @@ Apache2::Controller::Dispatch::Simple - simple dispatch mechanism for A2C
 
  package MyApp::Dispatch;
  use base qw(
-     Apache2::Controller::Dispatch
      Apache2::Controller::Dispatch::Simple
  );
 
- our %dispatch_map = (
+ # return hash reference from dispatch_map()
+ sub dispatch_map { {            
      foo            => 'MyApp::C::Foo',
      'foo/bar'      => 'MyApp::C::Foo::Bar',
- );
-
- sub handler { Apache2::Controller::Dispatch::handler(shift, __PACKAGE__) }
+ } }
 
 =head1 DESCRIPTION
 
 Implements find_controller() for Apache2::Controller::Dispatch with
 a simple URI-to-controller module mapping.  Your URI's are the keys
-of the %dispatch_map hash in your base package, and the values are
+of the C<< dispatch_map() >> hash in your base package, and the values are
 the Apache2::Controller modules to which those URI's should be dispatched.
 
 This dispatches URI's in a case-insensitive fashion.  It searches from
@@ -38,64 +45,22 @@ longest known path to shortest.  For a site with many controllers and
 paths, a trie could possibly be more efficient.  Consider that implementation
 for another Dispatch plugin module.
 
-Any implementation of find_controller() should throw an 
-L<Apache2::Controller::X> with http => Apache2::Const::NOT_FOUND in the
-event that the detected method selected does not appear in the list of
-C<@ALLOWED_METHODS> in the controller module.  
-See L<Apache2::Controller::Funk/check_allowed_method( )>
-
-Successful run of find_controller() should result in four items of
-data being set in request->notes and request->pnotes:
-
-=over 4
-
-=item notes->{relative_uri} = matching part of uri relative to location
-
-This is the uri relative to the location. For example,
-if the dispatch module is the init handler in a C<< <Location /subdir> >>
-config block, then for /subdir/foo/bar/biz/zip in this example code,
-relative_uri should be 'foo/bar' because this is the key of %dispatch_map
-that was matched.  /subdir/foo/bar is the 'virtual directory.'
-
-If there is no relative uri, for example if the uri requested was /subdir
-and this is the same as the location, then C<notes->{relative_uri}> would be set to 
-the empty string.
-
-=item notes->{controller} = selected package name
-
-This should be the name of an Apache2::Controller subclass selected
-for dispatch.
-
-=item notes->{method} = method name in controller to process the uri
-
-This is the name of the method of the controller to use for this request.
-
-=item pnotes->{path_args} = [ remaining path_info ]
-
-The remaining 'virtual directory' arguments of the uri.
-In the example above for notes->{relative_uri}, this is [ 'biz', 'zip' ].
-
-=back
-
-@path_args is the array of remaining elements.  For example if your
-dispatch map contains the URI 'foo', and the incoming URI was '/foo/bar/baz',
-then $r->pnotes->{path_args} should be ['bar', 'baz'] before returning.
-
 =head1 METHODS
 
 =cut
 
 use strict;
-use warnings;
+use warnings FATAL => 'all';
 use English '-no_match_vars';
 
+use base qw( Apache2::Controller::Dispatch );
+
 use Apache2::Controller::X;
 use Apache2::Controller::Funk qw( controller_allows_method check_allowed_method );
 
 use Log::Log4perl qw(:easy);
 use YAML::Syck;
 
-my %dispatch_maps   = ( );
 my %search_uris     = ( );
 my %uri_lengths     = ( );
 
@@ -103,23 +68,19 @@ my %uri_lengths     = ( );
 sub _get_class_info {
     my ($self) = @_;
     my $class = $self->{class};
-    my ($dispatch_map, $uri_length_map, $search_uri_list) = ();
-    if (exists $dispatch_maps{$class}) {
-        $dispatch_map       = $dispatch_maps{$class};
+    my $dispatch_map = $self->get_dispatch_map();
+    my ($uri_length_map, $search_uri_list) = ();
+    if (exists $uri_lengths{$class}) {
         $uri_length_map     = $uri_lengths{$class};
         $search_uri_list    = $search_uris{$class};
     }
     else {
-        # find the dispatch map in parent class if not yet cached in this module
-        eval '$dispatch_map = \%'.$self->{class}.'::dispatch_map';
-        $dispatch_maps{$class} = $dispatch_map;
-
         # search dispatch uri keys from longest to shortest
         my @uris = keys %{$dispatch_map};
-        Apache2::Controller::X->throw(
-            "Upper case characters not allowed in $class dispatch_map "
+
+        a2cx "Upper case characters not allowed in $class dispatch_map "
             ."when using ".__PACKAGE__." to dispatch URIs."
-        ) if grep m/ \p{IsUpper} /mxs, @uris;
+            if grep m/ \p{IsUpper} /mxs, @uris;
 
         $uri_length_map = $uri_lengths{$class} = { };
         $uri_length_map->{$_} = length $_ for @uris;
@@ -128,7 +89,6 @@ sub _get_class_info {
             sort { $uri_length_map->{$b} <=> $uri_length_map->{$a} } @uris 
         ];
 
-        DEBUG(sub{"dispatch_maps:".Dump(\%dispatch_maps)});
         DEBUG(sub{"search_uris:".Dump(\%search_uris)});
         DEBUG(sub{"uri_lengths:".Dump(\%uri_lengths)});
     }
@@ -200,9 +160,8 @@ sub find_controller {
             }
 
             $controller = $dispatch_map->{$search_uri} 
-                || Apache2::Controller::X->throw(
-                    "No controller assigned in $class dispatch map for $search_uri."
-                );
+                || a2cx
+                  "No controller assigned in $class dispatch map for $search_uri.";
             
             # extract the method and the rest of the path args from the uri
             if ($next_char) {
@@ -237,9 +196,10 @@ sub find_controller {
 
     if (!$controller) {
         DEBUG("No controller found.  Using default module from dispatch map.");
-        $controller = $dispatch_map->{default} || Apache2::Controller::X->throw(
-            "No 'default' controller assigned in $class dispatch map."
-        );
+
+        $controller = $dispatch_map->{default} 
+            || a2cx "No 'default' controller assigned in $class dispatch map.";
+
         my $first_arg;
         ($first_arg, @path_args) = split '/', $uri;
         if (controller_allows_method($controller => $first_arg)) {
@@ -251,7 +211,7 @@ sub find_controller {
         }
     }
 
-    Apache2::Controller::X->throw("No controller module found.") if !$controller;
+    a2cx "No controller module found." if !$controller;
 
     $method       ||= 'default';
     $relative_uri ||= '';
@@ -280,6 +240,8 @@ sub find_controller {
 
 L<Apache2::Controller::Dispatch>
 
+L<Apache2::Controller::Dispatch::HashTree>
+
 L<Apache2::Controller>
 
 =head1 AUTHOR
