@@ -1,7 +1,18 @@
+package Apache2::Controller::Funk;
+
 =head1 NAME
 
 Apache2::Controller::Funk
 
+=head1 VERSION
+
+Version 1.000.000 - FIRST RELEASE
+
+=cut
+
+use version;
+our $VERSION = version->new('1.000.000');
+
 =head1 SYNOPSIS
 
  $bool = controller_allows_method($class, $method); 
@@ -17,10 +28,8 @@ queried futher using 'exists', which is very fast.
 
 =cut
 
-package Apache2::Controller::Funk;
-
 use strict;
-use warnings FATAL => 'all', NONFATAL => 'redefine';
+use warnings FATAL => 'all';
 use English '-no_match_vars';
 
 use base 'Exporter';
@@ -38,18 +47,19 @@ our @EXPORT_OK = qw(
     controller_allows_method
     check_allowed_method
     log_bad_request_reason
+    default_consumer_secret
 );
 
 Readonly my $ACCESS_LOG_REASON_LENGTH => 60;
 
-
 =head1 IMPORTABLE FUNCTIONS
 
-=head2 controller_allows_method( )
+=head2 controller_allows_method
 
  $bool = controller_allows_method($class, $method); # controller_allows_method()
 
-Ask if method name is in @ALLOWED_METHODS in the given controller package.
+Ask if method name is returned by C<< allowed_methods() >>
+in the given controller package.
 
 Only two 'exists' calls are required for each query after caching the 
 first result for this child.
@@ -60,8 +70,8 @@ my %allowed_methods = ( );
 sub controller_allows_method {
     my ($class, $method) = @_;
 
-    Apache2::Controller::X->throw("class undefined")  if !defined $class;
-    Apache2::Controller::X->throw("method undefined") if !defined $method;
+    a2cx "class undefined"  if !defined $class;
+    a2cx "method undefined" if !defined $method;
     DEBUG(sub{
         "checking class '$class', method '$method', allowed is:\n"
         .Dump(\%allowed_methods)
@@ -73,21 +83,16 @@ sub controller_allows_method {
     if (!exists $allowed_methods{$class}) {
 
         eval "require $class;";
-        Apache2::Controller::X->throw("cannot require $class: $EVAL_ERROR")
-            if $EVAL_ERROR;
+        a2cx "cannot require $class: $EVAL_ERROR" if $EVAL_ERROR;
 
         my $isa_a2c; 
         eval "\$isa_a2c = $class->isa('Apache2::Controller');";
-        Apache2::Controller::X->throw("$class is not an Apache2::Controller")
-            unless $isa_a2c;
+        a2cx "$class is not an Apache2::Controller" unless $isa_a2c;
 
-        my @allowed_methods = ( );
-        my $stmt = '@allowed_methods = @'.$class.'::ALLOWED_METHODS;';
-        DEBUG("stmt: '$stmt'");
-        eval $stmt;
-        Apache2::Controller::X->throw(
-            "$class does not have \@ALLOWED_METHODS: $EVAL_ERROR"
-        ) if $EVAL_ERROR;
+        a2cx "$class knows no allowed_methods()"
+            unless $class->can('allowed_methods');
+
+        my @allowed_methods = $class->allowed_methods();
 
         DEBUG("allowed_methods: (@allowed_methods)");
         $allowed_methods{$class} = { map {($_=>1)} @allowed_methods };
@@ -96,27 +101,25 @@ sub controller_allows_method {
     return exists $allowed_methods{$class}{$method};
 }
 
-=head2 check_allowed_method( )
+=head2 check_allowed_method
 
  check_allowed_method($method, $class); # check_allowed_method()
 
 Throw a NOT_FOUND exception if the method is not an allowed method
-in the @ALLOWED_METHODS array in the controller package.
+in the C<< allowed_methods() >> list in the controller package.
 
 =cut
 
 sub check_allowed_method {
     my ($class, $method) = @_;
-    Apache2::Controller::X->throw("class undefined")  if !defined $class;
-    Apache2::Controller::X->throw("method undefined") if !defined $method;
+    a2cx "class undefined"  if !defined $class;
+    a2cx "method undefined" if !defined $method;
     DEBUG("checking class '$class', method '$method'");
 
     if (!controller_allows_method($class, $method)) {
         DEBUG("Method $method not allowed in $class.");
-        Apache2::Controller::X->throw(
-            message     => "Method $method not allowed from $class.",
-            status => Apache2::Const::NOT_FOUND,
-        );
+        a2cx message    => "Method $method not allowed from $class.",
+            status      => Apache2::Const::NOT_FOUND;
     }
     return;
 }
@@ -132,7 +135,7 @@ version of $X in case $X is too long.
 
 sub log_bad_request_reason {
     my ($r, $X) = @_;
-    Apache2::Controller::X->throw('usage: log_bad_request_reason($r, $X)') 
+    a2cx 'usage: log_bad_request_reason($r, $X)'
         if !$r || !ref($r) || !$r->can('log_reason') || !$X;
     
     my $x_text = "$X";
