@@ -5,6 +5,15 @@ package Apache2::Controller::NonResponseBase;
 Apache2::Controller::NonResponseBase - internal base class for 
 non-response handlers in Apache2::Controller framework
 
+=head1 VERSION
+
+Version 1.000.000 - FIRST RELEASE
+
+=cut
+
+use version;
+our $VERSION = version->new('1.000.000');
+
 =head1 SYNOPSIS
 
 This is an INTERNAL base class and you don't need to use it.
@@ -39,7 +48,7 @@ processing the chain of handlers... we'll see.
 =cut
 
 use strict;
-use warnings;
+use warnings FATAL => 'all';
 use English '-no_match_vars';
 
 use Log::Log4perl qw(:easy);
@@ -47,6 +56,7 @@ use YAML::Syck;
 
 use Apache2::RequestRec ();
 use Apache2::RequestUtil ();
+use Apache2::Log;
 use Apache2::Const -compile => qw( :common :http :methods );
 
 use Apache2::Controller::X;
@@ -69,22 +79,15 @@ sub handler : method {
 
     eval { 
         $handler = $class->new($r);
-        $handler->process(); 
+        $status = $handler->process(); 
     };
     if ($X = Exception::Class->caught('Apache2::Controller::X')) {
-        if ($X->isa('Apache2::Controller::X::Redirect')) {
-            $status = Apache2::Const::REDIRECT;
-            $r->err_headers_out->add(Location => "$X");
-            DEBUG("redirecting to $X");
-        }
-        else {
-            $status = $X->status || Apache2::Const::SERVER_ERROR;
-            DEBUG("Caught an Apache2::Controller::X: $status");
-            DEBUG(ref($X).": $X\n".($X->dump ? Dump($X->dump) : '').$X->trace());
-        }
+        $status = $X->status || Apache2::Const::SERVER_ERROR;
+        WARN("Caught an Apache2::Controller::X: $status");
+        WARN(ref($X).": $X\n".($X->dump ? Dump($X->dump) : '').$X->trace());
     }
     elsif ($X = $EVAL_ERROR) {
-        DEBUG("Caught an unknown error: $X");
+        WARN("Caught an unknown error: $X");
         $status = Apache2::Const::SERVER_ERROR;
     }
 
@@ -95,22 +98,24 @@ sub handler : method {
 
     if ($status && $status >= Apache2::Const::HTTP_BAD_REQUEST) {
         DEBUG("logging bad request");
-        log_bad_request_reason($r, $X);
-        return Apache2::Const::DONE;  # stop processing further handlers
-    }
-    elsif ($status && $status == Apache2::Const::REDIRECT) {
-        return Apache2::Const::DONE;
-    }
-    else {
-        DEBUG("returning OK");
-        return Apache2::Const::OK;
+        eval { log_bad_request_reason($r, $X); };
+        if (my $X = Exception::Class->caught('Apache2::Controller::X')) {
+            FATAL("Bad error logging bad request! '$X'\n".$X->trace);
+        }
+        elsif ($EVAL_ERROR) {
+            FATAL("Weird error logging bad request! '$EVAL_ERROR'");
+        }
     }
+    $status = Apache2::Const::OK if !defined $status;
+    DEBUG("returning '$status'");
+    return $status;
 }
 
 =head2 new
 
 C<new()> creates an object of the child class and assigns the
-C<<Apache2::RequestRec>> object to C<<$self->{r}>>.
+C<< Apache2::RequestRec >> object to 
+C<< $self->{r} >>.
 
 If the parent class defines a method C<init()>, this will
 be called at the end of object creation.
@@ -118,10 +123,13 @@ be called at the end of object creation.
 Unlike L<Apache2::Controller>, the handler object of other handlers
 that use this package as a base do not create, delegate to and subclass
 the L<Apache2::Request> object.  They just keep the original 
-L<Apache2::RequestRec> object in C<<$self->{r}>>.
+L<Apache2::RequestRec> object in 
+C<< $self->{r} >>.
 
 =cut
 
+my %can_init;
+
 sub new {
     my ($class, $r) = @_;
 
@@ -133,7 +141,8 @@ sub new {
     };
     bless $self, $class;
 
-    $self->init() if $self->can('init');
+    $can_init{$class} = $self->can('init') if !exists $can_init{$class};
+    $self->init() if $can_init{$class};
 
     return $self;
 }
@@ -142,8 +151,12 @@ sub new {
 
 =head1 SEE ALSO
 
-L<Apache2::Controller::Dispatch>,
+L<Apache2::Controller::NonResponseRequest>
+
+L<Apache2::Controller::Dispatch>
+
 L<Apache2::Controller::Session>
+
 L<Apache2::Controller>
 
 =head1 AUTHOR
