@@ -4,24 +4,38 @@ package Apache2::Controller::Render::Template;
 
 Apache2::Controller::Render::Template - A2C render() with Template Toolkit
 
+=head1 VERSION
+
+Version 1.000.000 - FIRST RELEASE
+
+=cut
+
+use version;
+our $VERSION = version->new('1.000.000');
+
 =head1 SYNOPSIS
 
  # apache2 config file
 
- PerlModule Apache2::Controller::Directives
+ PerlLoadModule Apache2::Controller::Directives
+ PerlLoadModule Apache2::Controller::DBI::Connector
 
  # location of templates - must be defined
- A2CRenderTemplateDir           /var/myapp/templates
+ A2C_Render_Template_Path  /var/myapp/templates /var/myapp/template_components
 
- # looks for templates in /var/myapp/templates/foo/
  <Location /foo>
     SetHandler modperl
-    PerlInitHandler MyApp::Dispatch::Foo
+    PerlInitHandler             MyApp::Dispatch::Foo
+
+    # set directives A2C_DBI_DSN, etc. 
+    PerlHeaderParserHandler     Apache2::Controller::DBI::Connector
  </Location>
 
- # see L<Apache2::Controller::Dispatch> for A2C Dispatch inheritance.
+See L<Apache2::Controller::Dispatch> for A2C Dispatch implementations.
 
- package MyApp::C::Foo;  # let's assume this controller was dispatched
+See L<Apache2::Controller::Directives> and L<Apache2::Controller::DBI::Connector>.
+
+ package MyApp::C::Bar;  # let's assume this controller was dispatched
 
  use strict;
  use warnings;
@@ -29,23 +43,17 @@ Apache2::Controller::Render::Template - A2C render() with Template Toolkit
  use base qw(
     Apache2::Controller
     Apache2::Controller::Render::Template
-    MyApp::Model::Methods
-    MyApp::Security
  );
 
  use Apache2::Const -complie => qw( OK );
 
- my @ALLOWED_METHODS = qw( default );
-
- # suppose MyApp::Model::Methods connects dbh in startup sequence
- # and suppose MyApp::Security implements my_detaint_path_args()
- # to call a branch function for 'name' that detaints [ last, first ]
+ sub allowed_methods {qw( default )}
 
  sub default {
-    my ($self) = @_;
+    my ($self, @first, @last) = @_;
     my @path_args = $self->my_detaint_path_args('name'); # from $self->{path_args}
 
-    $self->{stash}{creditcards} = $self->{dbh}->fetchall_arrayref(
+    $self->{stash}{creditcards} = $self->pnotes->{dbh}->fetchall_arrayref(
         q{  SELECT ccnum, exp, addr1, zip, cac 
             FROM customer_credit_cards 
             WHERE lname = ? AND fname = ?
@@ -77,6 +85,42 @@ Apache2::Controller::Render::Template - A2C render() with Template Toolkit
 
 This module provides a nice rendering mechanism for Apache2::Controller.
 
+=head1 TEMPLATE OPTIONS
+
+You can specify options for L<Template> in one of two ways:
+
+=head2 DIRECTIVES
+
+By using 
+L<Apache2::Controller::Directives/A2C_Render_Template_Opts>:
+
+ <Location '/foo'>
+     A2C_Render_Template_Opts INTERPOLATE 1
+     A2C_Render_Template_Opts PRE_PROCESS header
+     A2C_Render_Template_Opts POST_CHOMP  1
+ </Location>
+
+=head2 METHOD template_options()
+
+Or by implementing C<<template_options>> in your controller:
+
+ package MyApp::Controller;
+ use base qw( Apache2::Controller Apache2::Controller::Render::Template );
+ sub allowed_methods {qw( default )}
+
+ sub template_options {
+     my ($self) = @_;
+     return {
+         INTERPOLATE => 1,
+         PRE_PROCESS => 'header',
+         POST_CHOMP  =. 1,
+     };
+ }
+
+ sub default { 
+    # ...
+ }
+
 =head1 STASH FUNCTIONS
 
 Several subroutine references are automatically included
@@ -166,8 +210,7 @@ sub render {
 
     # assimilate output to a scalar 
     my $output;
-    $tt->process($template, $self->{stash}, \$output) 
-        || Apache2::Controller::X->throw($tt->error());
+    $tt->process($template, $self->{stash}, \$output) || a2cx $tt->error();
 
     $self->print($output);
 
@@ -211,8 +254,7 @@ sub render_fast {
 
     my $tt = $self->get_tt_obj();
     # pass Apache2::Request object to print directly.
-    $tt->process($template, $self->{stash}, $self->{r}) 
-        || Apache2::Controller::X->throw($tt->error());
+    $tt->process($template, $self->{stash}, $self->{r}) || a2cx $tt->error();
 
     return;
 }
@@ -280,8 +322,8 @@ sub error {
     $self->{stash}{status_line} = $status_line;
     $self->{stash}{status} = $status;
 
-    my $template_dir = $self->get_directive('A2CRenderTemplateDir')
-        || Apache2::Controller::X->throw('A2CRenderTemplateDir not defined.');
+    my $template_dir = $self->get_directive('A2C_Render_Template_Path')
+        || a2cx 'A2C_Render_Template_Path not defined.';
     if (exists $error_templates{$template_dir}{$status_file}) {
 
         my $template = $error_templates{$template_dir}{$status_file};
@@ -293,9 +335,7 @@ sub error {
                 $X->rethrow();
             }
             else {
-                Apache2::Controller::X->throw(
-                    "Cannot process any template for unknown-type error: $X"
-                );
+                a2cx "Cannot process any template for unknown-type error: $X";
             }
         }
 
@@ -325,12 +365,10 @@ sub error {
                 }
                 else {
                     my $dump = { tries => \%try_errors, reftype => ref $X };
-                    Apache2::Controller::X->throw(
-                        message     => "$X",
-                        status => $status,
+                    a2cx message    => "$X",
+                        status      => $status,
                         status_line => $status_line,
-                        'dump'      => $dump,
-                    );
+                        'dump'      => $dump;
                 }
             }
         }
@@ -350,12 +388,12 @@ To override the auto-select template, just set $self->{template}
 before you call C<<render()>>.
 
 It looks for templates in a computed directory.  The directory where it
-looks will always be the directory set with the A2CRenderTemplateDir 
+looks will always be the directory set with the A2C_Render_Template_Path 
 directive in the config file, appended with the current request location,
 i.e. the directory of the Location directive in the config file, 
 appended with relative_uri, appended with method name and '.html'.
 
- A2CRenderTemplateDir + location + relative_uri + method.html
+ A2C_Render_Template_Path + location + relative_uri + method.html
 
 For example, the sequence in SYNOPSIS above renders the file 
 C</var/myapp/templates/foo/default.html> .
@@ -409,8 +447,7 @@ sub detect_template {
     }
 
     (my $rel_uri = $self->notes->{relative_uri}) =~ s{ \A / }{}mxs;
-    Apache2::Controller::X->throw('notes->{relative_uri} not set')
-        if !defined $rel_uri;
+    a2cx 'notes->{relative_uri} not set' if !defined $rel_uri;
 
     my $file = "$self->{method}.html";
 
@@ -425,8 +462,7 @@ sub detect_template {
         ? File::Spec->catfile( $loc, $rel_uri, $file )
         : File::Spec->catfile( $loc, $file );
 
-    Apache2::Controller::X->throw("bad template path $_")
-        if $template =~ m{ \.\. / }mxs;
+    a2cx "bad template path $_" if $template =~ m{ \.\. / }mxs;
 
     DEBUG("Detected self->{template} to be '$template'");
 
@@ -434,89 +470,49 @@ sub detect_template {
     return $template;
 }
 
-=head2 detect_template_include_path
-
-Like C<<detect_template()>>, this detects the appropriate include
-path for the template toolkit object, sets it as 
-C<<$self->{template_include_path}>> and returns it.
-
-This is the directive A2CRenderTemplateDir plus the C<< <Location> >>.
-So if you set up your handler to process everything under location /render,
-and A2CRenderTemplateDir was /var/www/mytemplates, the include_path
-used for the TT object would be /var/www/mytemplates/render/.
-
-=cut
-
-sub detect_template_include_path {
-    my ($self) = @_;
-
-    return $self->{template_include_path} if $self->{template_include_path};
-
-    my $template_path = $self->get_directive('A2CRenderTemplateDir')
-        || Apache2::Controller::X->throw("A2CRenderTemplateDir not defined");
-
-    my $loc = $self->location();
-
-    # trim leading and trailing /'s
-    $loc = '' if $loc eq '/';
-    if ($loc) {
-        $loc = substr($loc, 1) if length $loc > 1 && substr($loc, 0, 1) eq '/';
-        substr($loc, -1, 1, '') if substr($loc, -1) eq '/';
-    }
-    DEBUG("so far have '$template_path' / '$loc'");
-
-    DEBUG(sub{Dump({
-        loc     => $loc,
-        template_path => $template_path,
-    })});
-
-    my $dir = $self->{template_include_path} = File::Spec->catfile(
-        $template_path,
-        $loc,
-    );
-
-    return $dir;
-}
-
 =head2 get_tt_obj
 
 Get the L<Template> object set up with the appropriate include directory
-set from the directive C<<A2CRenderTemplateDir>>.
+set from the directive C<<A2C_Render_Template_Path>>.
 
-Directive C<A2CRenderTemplateOpts> sets default C<new()>
+Directive C<A2C_Render_Template_Opts> sets default C<new()>
 options for L<Template>.  
 
 =cut
 
+my %implements_template_opts;
 my %tts = ();
 sub get_tt_obj {
     my ($self) = @_;
-  # my $include_path = $self->detect_template_include_path();
 
-    my $include_path = $self->get_directive('A2CRenderTemplateDir')
-        || Apache2::Controller::X->throw("A2CRenderTemplateDir not defined");
+    my $include_path = $self->get_directive('A2C_Render_Template_Path')
+        || a2cx "A2C_Render_Template_Path not defined";
 
-    return $tts{$include_path} if exists $tts{$include_path};
+    my $class = $self->{class};
+    $implements_template_opts{$class} = $self->can('template_opts')
+        if !exists $implements_template_opts{$class};
 
-    DEBUG("using include_path '$include_path' to set up a new TT object");
+    my %opts 
+        = $implements_template_opts{$class}
+        ? %{ $self->template_opts() }
+        : %{ $self->get_directive('A2C_Render_Template_Opts') || { } };
 
-    my %opts = %{ $self->get_directive('A2CRenderTemplateOpts') || { } };
     $opts{INCLUDE_PATH} 
         = !$opts{INCLUDE_PATH}    ? $include_path
-        : ref $opts{INCLUDE_PATH} ? [@{$opts{INCLUDE_PATH}}, $include_path]
-        :                           [$opts{INCLUDE_PATH}, $include_path];
+        : ref $opts{INCLUDE_PATH} ? [@{$opts{INCLUDE_PATH}}, @{$include_path}]
+        :                           [$opts{INCLUDE_PATH}, @{$include_path}];
 
-  # $opts{DEBUG} = 'all';   # turn on if L4P set to DEBUG here??
+    my $opts_key = Dump(\%opts);
+    DEBUG("using opts for new TT on $include_path:\n$opts_key");
 
-    DEBUG(sub{"using opts for new TT on $include_path:\n".Dump(\%opts)});
+    return $tts{$opts_key} if exists $tts{$opts_key};
 
     my $tt;
     
     eval { $tt = Template->new(\%opts) || die $Template::ERROR."\n" };
-    Apache2::Controller::X->throw("No TT for $include_path: $EVAL_ERROR") 
-        if $EVAL_ERROR;
+    a2cx "No TT for $include_path: $EVAL_ERROR" if $EVAL_ERROR;
 
-    $tts{$include_path} = $tt;
+    $tts{$opts_key} = $tt;
 
     return $tt;
 }
