@@ -1,6 +1,6 @@
 package TestApp::HashTree::Biz;
 use base qw( Apache2::Controller );
-our @ALLOWED_METHODS = qw( default );
+sub allowed_methods {qw( default )}
 sub default {
 }
 1;
